@@ -35,6 +35,16 @@ using namespace KWayland::Client;
 using namespace KWayland::Server;
 
 Q_DECLARE_METATYPE(Qt::MouseButton)
+typedef void (KWayland::Client::PlasmaWindow::*ClientWindowSignal)();
+Q_DECLARE_METATYPE(ClientWindowSignal)
+typedef void (KWayland::Server::PlasmaWindowInterface::*ServerWindowBoolSetter)(bool);
+Q_DECLARE_METATYPE(ServerWindowBoolSetter)
+typedef void (KWayland::Server::PlasmaWindowInterface::*ServerWindowStringSetter)(const QString&);
+Q_DECLARE_METATYPE(ServerWindowStringSetter)
+typedef void (KWayland::Server::PlasmaWindowInterface::*ServerWindowQuint32Setter)(quint32);
+Q_DECLARE_METATYPE(ServerWindowQuint32Setter)
+typedef void (KWayland::Server::PlasmaWindowInterface::*ServerWindowVoidSetter)();
+Q_DECLARE_METATYPE(ServerWindowVoidSetter)
 
 class PlasmaWindowModelTest : public QObject
 {
@@ -60,19 +70,26 @@ private Q_SLOTS:
     void testIsOnAllDesktops();
     void testIsDemandingAttention();
     void testSkipTaskbar();
+    void testSkipSwitcher();
     void testIsShadeable();
     void testIsShaded();
     void testIsMovable();
     void testIsResizable();
     void testIsVirtualDesktopChangeable();
     void testIsCloseable();
+    void testGeometry();
     void testTitle();
     void testAppId();
+    void testPid();
     void testVirtualDesktop();
     // TODO icon: can we ensure a theme is installed on CI?
     void testRequests();
     // TODO: minimized geometry
     // TODO: model reset
+    void testCreateWithUnmappedWindow();
+    void testChangeWindowAfterModelDestroy_data();
+    void testChangeWindowAfterModelDestroy();
+    void testCreateWindowAfterModelDestroy();
 
 private:
     bool testBooleanData(PlasmaWindowModel::AdditionalRoles role, void (PlasmaWindowInterface::*function)(bool));
@@ -173,9 +190,6 @@ if (!QTest::qCompare(actual, expected, #actual, #expected, __FILE__, __LINE__))\
     m_display->dispatchEvents();
     QSignalSpy dataChangedSpy(model, &PlasmaWindowModel::dataChanged);
     VERIFY(dataChangedSpy.isValid());
-    // just creating sends one changed, this could be improved in the protocol
-    VERIFY(dataChangedSpy.wait());
-    dataChangedSpy.clear();
 
     const QModelIndex index = model->index(0);
     COMPARE(model->data(index, role).toBool(), false);
@@ -208,6 +222,7 @@ void PlasmaWindowModelTest::testRoleNames_data()
     QTest::newRow("decoration") << int(Qt::DecorationRole) << QByteArrayLiteral("DecorationRole");
 
     QTest::newRow("AppId")                << int(PlasmaWindowModel::AppId) << QByteArrayLiteral("AppId");
+    QTest::newRow("Pid")                  << int(PlasmaWindowModel::Pid) << QByteArrayLiteral("Pid");
     QTest::newRow("IsActive")             << int(PlasmaWindowModel::IsActive) << QByteArrayLiteral("IsActive");
     QTest::newRow("IsFullscreenable")     << int(PlasmaWindowModel::IsFullscreenable) << QByteArrayLiteral("IsFullscreenable");
     QTest::newRow("IsFullscreen")         << int(PlasmaWindowModel::IsFullscreen) << QByteArrayLiteral("IsFullscreen");
@@ -221,12 +236,14 @@ void PlasmaWindowModelTest::testRoleNames_data()
     QTest::newRow("IsOnAllDesktops")      << int(PlasmaWindowModel::IsOnAllDesktops) << QByteArrayLiteral("IsOnAllDesktops");
     QTest::newRow("IsDemandingAttention") << int(PlasmaWindowModel::IsDemandingAttention) << QByteArrayLiteral("IsDemandingAttention");
     QTest::newRow("SkipTaskbar")          << int(PlasmaWindowModel::SkipTaskbar) << QByteArrayLiteral("SkipTaskbar");
+    QTest::newRow("SkipSwitcher")         << int(PlasmaWindowModel::SkipSwitcher) << QByteArrayLiteral("SkipSwitcher");
     QTest::newRow("IsShadeable")           << int(PlasmaWindowModel::IsShadeable) << QByteArrayLiteral("IsShadeable");
     QTest::newRow("IsShaded")             << int(PlasmaWindowModel::IsShaded) << QByteArrayLiteral("IsShaded");
     QTest::newRow("IsMovable")            << int(PlasmaWindowModel::IsMovable) << QByteArrayLiteral("IsMovable");
     QTest::newRow("IsResizable")          << int(PlasmaWindowModel::IsResizable) << QByteArrayLiteral("IsResizable");
     QTest::newRow("IsVirtualDesktopChangeable") << int(PlasmaWindowModel::IsVirtualDesktopChangeable) << QByteArrayLiteral("IsVirtualDesktopChangeable");
     QTest::newRow("IsCloseable")          << int(PlasmaWindowModel::IsCloseable) << QByteArrayLiteral("IsCloseable");
+    QTest::newRow("Geometry")          << int(PlasmaWindowModel::Geometry) << QByteArrayLiteral("Geometry");
 }
 
 void PlasmaWindowModelTest::testRoleNames()
@@ -268,10 +285,13 @@ void PlasmaWindowModelTest::testAddRemoveRows()
     // that index doesn't have children
     QCOMPARE(model->rowCount(model->index(0)), 0);
 
+    // process events in order to ensure that the resource is created on server side before we unmap the window
+    QCoreApplication::instance()->processEvents(QEventLoop::WaitForMoreEvents);
+
     // now let's remove that again
     QSignalSpy rowRemovedSpy(model, &PlasmaWindowModel::rowsRemoved);
     QVERIFY(rowRemovedSpy.isValid());
-    delete w;
+    w->unmap();
     QVERIFY(rowRemovedSpy.wait());
     QCOMPARE(rowRemovedSpy.count(), 1);
     QVERIFY(!rowRemovedSpy.first().at(0).toModelIndex().isValid());
@@ -281,6 +301,10 @@ void PlasmaWindowModelTest::testAddRemoveRows()
     // now the model is empty again
     QCOMPARE(model->rowCount(), 0);
     QVERIFY(!model->index(0).isValid());
+
+    QSignalSpy wDestroyedSpy(w, &QObject::destroyed);
+    QVERIFY(wDestroyedSpy.isValid());
+    QVERIFY(wDestroyedSpy.wait());
 }
 
 void PlasmaWindowModelTest::testDefaultData_data()
@@ -310,7 +334,9 @@ void PlasmaWindowModelTest::testDefaultData_data()
     QTest::newRow("IsMovable")            << int(PlasmaWindowModel::IsMovable) << QVariant(false);
     QTest::newRow("IsResizable")          << int(PlasmaWindowModel::IsResizable) << QVariant(false);
     QTest::newRow("IsVirtualDesktopChangeable") << int(PlasmaWindowModel::IsVirtualDesktopChangeable) << QVariant(false);
-    QTest::newRow("IsCloseable")           << int(PlasmaWindowModel::IsCloseable) << QVariant(false);
+    QTest::newRow("IsCloseable")          << int(PlasmaWindowModel::IsCloseable) << QVariant(false);
+    QTest::newRow("Geometry")             << int(PlasmaWindowModel::Geometry) << QVariant(QRect());
+    QTest::newRow("Pid")                  << int(PlasmaWindowModel::Pid) << QVariant(0);
 }
 
 void PlasmaWindowModelTest::testDefaultData()
@@ -390,6 +416,11 @@ void PlasmaWindowModelTest::testSkipTaskbar()
     QVERIFY(testBooleanData(PlasmaWindowModel::SkipTaskbar, &PlasmaWindowInterface::setSkipTaskbar));
 }
 
+void PlasmaWindowModelTest::testSkipSwitcher()
+{
+    QVERIFY(testBooleanData(PlasmaWindowModel::SkipSwitcher, &PlasmaWindowInterface::setSkipSwitcher));
+}
+
 void PlasmaWindowModelTest::testIsShadeable()
 {
     QVERIFY(testBooleanData(PlasmaWindowModel::IsShadeable, &PlasmaWindowInterface::setShadeable));
@@ -420,6 +451,36 @@ void PlasmaWindowModelTest::testIsCloseable()
     QVERIFY(testBooleanData(PlasmaWindowModel::IsCloseable, &PlasmaWindowInterface::setCloseable));
 }
 
+void PlasmaWindowModelTest::testGeometry()
+{
+    auto model = m_pw->createWindowModel();
+    QVERIFY(model);
+
+    QSignalSpy rowInsertedSpy(model, &PlasmaWindowModel::rowsInserted);
+    QVERIFY(rowInsertedSpy.isValid());
+
+    auto w = m_pwInterface->createWindow(m_pwInterface);
+    QVERIFY(w);
+    QVERIFY(rowInsertedSpy.wait());
+
+    const QModelIndex index = model->index(0);
+
+    QCOMPARE(model->data(index, PlasmaWindowModel::Geometry).toRect(), QRect());
+
+    QSignalSpy dataChangedSpy(model, &PlasmaWindowModel::dataChanged);
+    QVERIFY(dataChangedSpy.isValid());
+
+    const QRect geom(0, 15, 50, 75);
+    w->setGeometry(geom);
+
+    QVERIFY(dataChangedSpy.wait());
+    QCOMPARE(dataChangedSpy.count(), 1);
+    QCOMPARE(dataChangedSpy.last().first().toModelIndex(), index);
+    QCOMPARE(dataChangedSpy.last().last().value<QVector<int>>(), QVector<int>{int(PlasmaWindowModel::Geometry)});
+
+    QCOMPARE(model->data(index, PlasmaWindowModel::Geometry).toRect(), geom);
+}
+
 void PlasmaWindowModelTest::testTitle()
 {
     auto model = m_pw->createWindowModel();
@@ -433,9 +494,6 @@ void PlasmaWindowModelTest::testTitle()
     m_display->dispatchEvents();
     QSignalSpy dataChangedSpy(model, &PlasmaWindowModel::dataChanged);
     QVERIFY(dataChangedSpy.isValid());
-    // just creating sends one changed, this could be improved in the protocol
-    QVERIFY(dataChangedSpy.wait());
-    dataChangedSpy.clear();
 
     const QModelIndex index = model->index(0);
     QCOMPARE(model->data(index, Qt::DisplayRole).toString(), QString());
@@ -461,9 +519,6 @@ void PlasmaWindowModelTest::testAppId()
     m_display->dispatchEvents();
     QSignalSpy dataChangedSpy(model, &PlasmaWindowModel::dataChanged);
     QVERIFY(dataChangedSpy.isValid());
-    // just creating sends one changed, this could be improved in the protocol
-    QVERIFY(dataChangedSpy.wait());
-    dataChangedSpy.clear();
 
     const QModelIndex index = model->index(0);
     QCOMPARE(model->data(index, PlasmaWindowModel::AppId).toString(), QString());
@@ -476,6 +531,24 @@ void PlasmaWindowModelTest::testAppId()
     QCOMPARE(model->data(index, PlasmaWindowModel::AppId).toString(), QStringLiteral("org.kde.testapp"));
 }
 
+void PlasmaWindowModelTest::testPid()
+{
+    auto model = m_pw->createWindowModel();
+    QVERIFY(model);
+    QSignalSpy rowInsertedSpy(model, &PlasmaWindowModel::rowsInserted);
+    QVERIFY(rowInsertedSpy.isValid());
+    auto w = m_pwInterface->createWindow(m_pwInterface);
+    w->setPid(1337);
+    QVERIFY(w);
+    m_connection->flush();
+    m_display->dispatchEvents();
+    QVERIFY(rowInsertedSpy.wait());
+
+    //pid should be set as soon as the new row appears
+    const QModelIndex index = model->index(0);
+    QCOMPARE(model->data(index, PlasmaWindowModel::Pid).toInt(), 1337);
+}
+
 void PlasmaWindowModelTest::testVirtualDesktop()
 {
     auto model = m_pw->createWindowModel();
@@ -489,9 +562,6 @@ void PlasmaWindowModelTest::testVirtualDesktop()
     m_display->dispatchEvents();
     QSignalSpy dataChangedSpy(model, &PlasmaWindowModel::dataChanged);
     QVERIFY(dataChangedSpy.isValid());
-    // just creating sends one changed, this could be improved in the protocol
-    QVERIFY(dataChangedSpy.wait());
-    dataChangedSpy.clear();
 
     const QModelIndex index = model->index(0);
     QCOMPARE(model->data(index, PlasmaWindowModel::VirtualDesktop).toInt(), 0);
@@ -529,6 +599,10 @@ void PlasmaWindowModelTest::testRequests()
     QVERIFY(resizeRequestedSpy.isValid());
     QSignalSpy virtualDesktopRequestedSpy(w, &PlasmaWindowInterface::virtualDesktopRequested);
     QVERIFY(virtualDesktopRequestedSpy.isValid());
+    QSignalSpy keepAboveRequestedSpy(w, &PlasmaWindowInterface::keepAboveRequested);
+    QVERIFY(keepAboveRequestedSpy.isValid());
+    QSignalSpy keepBelowRequestedSpy(w, &PlasmaWindowInterface::keepBelowRequested);
+    QVERIFY(keepBelowRequestedSpy.isValid());
     QSignalSpy minimizedRequestedSpy(w, &PlasmaWindowInterface::minimizedRequested);
     QVERIFY(minimizedRequestedSpy.isValid());
     QSignalSpy maximizeRequestedSpy(w, &PlasmaWindowInterface::maximizedRequested);
@@ -540,6 +614,8 @@ void PlasmaWindowModelTest::testRequests()
     model->requestActivate(-1);
     model->requestClose(-1);
     model->requestVirtualDesktop(-1, 1);
+    model->requestToggleKeepAbove(-1);
+    model->requestToggleKeepBelow(-1);
     model->requestToggleMinimized(-1);
     model->requestToggleMaximized(-1);
     model->requestActivate(1);
@@ -547,6 +623,8 @@ void PlasmaWindowModelTest::testRequests()
     model->requestMove(1);
     model->requestResize(1);
     model->requestVirtualDesktop(1, 1);
+    model->requestToggleKeepAbove(1);
+    model->requestToggleKeepBelow(1);
     model->requestToggleMinimized(1);
     model->requestToggleMaximized(1);
     model->requestToggleShaded(1);
@@ -619,6 +697,30 @@ void PlasmaWindowModelTest::testRequests()
     QCOMPARE(minimizedRequestedSpy.count(), 0);
     QCOMPARE(maximizeRequestedSpy.count(), 0);
     QCOMPARE(shadeRequestedSpy.count(), 0);
+    // keep above
+    model->requestToggleKeepAbove(0);
+    QVERIFY(keepAboveRequestedSpy.wait());
+    QCOMPARE(keepAboveRequestedSpy.count(), 1);
+    QCOMPARE(keepAboveRequestedSpy.first().first().toBool(), true);
+    QCOMPARE(activateRequestedSpy.count(), 1);
+    QCOMPARE(closeRequestedSpy.count(), 1);
+    QCOMPARE(moveRequestedSpy.count(), 1);
+    QCOMPARE(resizeRequestedSpy.count(), 1);
+    QCOMPARE(virtualDesktopRequestedSpy.count(), 1);
+    QCOMPARE(maximizeRequestedSpy.count(), 0);
+    QCOMPARE(shadeRequestedSpy.count(), 0);
+    // keep Below
+    model->requestToggleKeepBelow(0);
+    QVERIFY(keepBelowRequestedSpy.wait());
+    QCOMPARE(keepBelowRequestedSpy.count(), 1);
+    QCOMPARE(keepBelowRequestedSpy.first().first().toBool(), true);
+    QCOMPARE(activateRequestedSpy.count(), 1);
+    QCOMPARE(closeRequestedSpy.count(), 1);
+    QCOMPARE(moveRequestedSpy.count(), 1);
+    QCOMPARE(resizeRequestedSpy.count(), 1);
+    QCOMPARE(virtualDesktopRequestedSpy.count(), 1);
+    QCOMPARE(maximizeRequestedSpy.count(), 0);
+    QCOMPARE(shadeRequestedSpy.count(), 0);
     // minimize
     model->requestToggleMinimized(0);
     QVERIFY(minimizedRequestedSpy.wait());
@@ -658,6 +760,20 @@ void PlasmaWindowModelTest::testRequests()
     // the toggles can also support a different state
     QSignalSpy dataChangedSpy(model, &PlasmaWindowModel::dataChanged);
     QVERIFY(dataChangedSpy.isValid());
+    // keepAbove
+    w->setKeepAbove(true);
+    QVERIFY(dataChangedSpy.wait());
+    model->requestToggleKeepAbove(0);
+    QVERIFY(keepAboveRequestedSpy.wait());
+    QCOMPARE(keepAboveRequestedSpy.count(), 2);
+    QCOMPARE(keepAboveRequestedSpy.last().first().toBool(), false);
+    // keepBelow
+    w->setKeepBelow(true);
+    QVERIFY(dataChangedSpy.wait());
+    model->requestToggleKeepBelow(0);
+    QVERIFY(keepBelowRequestedSpy.wait());
+    QCOMPARE(keepBelowRequestedSpy.count(), 2);
+    QCOMPARE(keepBelowRequestedSpy.last().first().toBool(), false);
     // minimize
     w->setMinimized(true);
     QVERIFY(dataChangedSpy.wait());
@@ -681,5 +797,117 @@ void PlasmaWindowModelTest::testRequests()
     QCOMPARE(shadeRequestedSpy.last().first().toBool(), false);
 }
 
+void PlasmaWindowModelTest::testCreateWithUnmappedWindow()
+{
+    // this test verifies that creating the model just when an unmapped window exists doesn't cause problems
+    // that is the unmapped window should be added (as expected), but also be removed again
+
+    // create a window in "normal way"
+    QSignalSpy windowCreatedSpy(m_pw, &PlasmaWindowManagement::windowCreated);
+    QVERIFY(windowCreatedSpy.isValid());
+    auto w = m_pwInterface->createWindow(m_pwInterface);
+    QVERIFY(w);
+    QVERIFY(windowCreatedSpy.wait());
+    PlasmaWindow *window = windowCreatedSpy.first().first().value<PlasmaWindow*>();
+    QVERIFY(window);
+    // make sure the resource is properly created on server side
+    QCoreApplication::instance()->processEvents(QEventLoop::WaitForMoreEvents);
+
+    QSignalSpy unmappedSpy(window, &PlasmaWindow::unmapped);
+    QVERIFY(unmappedSpy.isValid());
+    QSignalSpy destroyedSpy(window, &PlasmaWindow::destroyed);
+    QVERIFY(destroyedSpy.isValid());
+    // unmap should be triggered, but not yet the destroyed
+    w->unmap();
+    QVERIFY(unmappedSpy.wait());
+    QVERIFY(destroyedSpy.isEmpty());
+
+    auto model = m_pw->createWindowModel();
+    QVERIFY(model);
+    QCOMPARE(model->rowCount(), 1);
+    QSignalSpy rowRemovedSpy(model, &PlasmaWindowModel::rowsRemoved);
+    QVERIFY(rowRemovedSpy.isValid());
+    QVERIFY(rowRemovedSpy.wait());
+    QCOMPARE(rowRemovedSpy.count(), 1);
+    QCOMPARE(model->rowCount(), 0);
+    QCOMPARE(destroyedSpy.count(), 1);
+}
+
+void PlasmaWindowModelTest::testChangeWindowAfterModelDestroy_data()
+{
+    QTest::addColumn<ClientWindowSignal>("changedSignal");
+    QTest::addColumn<QVariant>("setter");
+    QTest::addColumn<QVariant>("value");
+
+    QTest::newRow("active")           << &PlasmaWindow::activeChanged                   << qVariantFromValue(&PlasmaWindowInterface::setActive)                   << QVariant(true);
+    QTest::newRow("minimized")        << &PlasmaWindow::minimizedChanged                << qVariantFromValue(&PlasmaWindowInterface::setMinimized)                << QVariant(true);
+    QTest::newRow("fullscreen")       << &PlasmaWindow::fullscreenChanged               << qVariantFromValue(&PlasmaWindowInterface::setFullscreen)               << QVariant(true);
+    QTest::newRow("keepAbove")        << &PlasmaWindow::keepAboveChanged                << qVariantFromValue(&PlasmaWindowInterface::setKeepAbove)                << QVariant(true);
+    QTest::newRow("keepBelow")        << &PlasmaWindow::keepBelowChanged                << qVariantFromValue(&PlasmaWindowInterface::setKeepBelow)                << QVariant(true);
+    QTest::newRow("maximized")        << &PlasmaWindow::maximizedChanged                << qVariantFromValue(&PlasmaWindowInterface::setMaximized)                << QVariant(true);
+    QTest::newRow("demandsAttention") << &PlasmaWindow::demandsAttentionChanged         << qVariantFromValue(&PlasmaWindowInterface::setDemandsAttention)         << QVariant(true);
+    QTest::newRow("closeable")        << &PlasmaWindow::closeableChanged                << qVariantFromValue(&PlasmaWindowInterface::setCloseable)                << QVariant(true);
+    QTest::newRow("minimizeable")     << &PlasmaWindow::minimizeableChanged             << qVariantFromValue(&PlasmaWindowInterface::setMinimizeable)             << QVariant(true);
+    QTest::newRow("maximizeable")     << &PlasmaWindow::maximizeableChanged             << qVariantFromValue(&PlasmaWindowInterface::setMaximizeable)             << QVariant(true);
+    QTest::newRow("fullscreenable")   << &PlasmaWindow::fullscreenableChanged           << qVariantFromValue(&PlasmaWindowInterface::setFullscreenable)           << QVariant(true);
+    QTest::newRow("skipTaskbar")      << &PlasmaWindow::skipTaskbarChanged              << qVariantFromValue(&PlasmaWindowInterface::setSkipTaskbar)              << QVariant(true);
+    QTest::newRow("shadeable")        << &PlasmaWindow::shadeableChanged                << qVariantFromValue(&PlasmaWindowInterface::setShadeable)                << QVariant(true);
+    QTest::newRow("shaded")           << &PlasmaWindow::shadedChanged                   << qVariantFromValue(&PlasmaWindowInterface::setShaded)                   << QVariant(true);
+    QTest::newRow("movable")          << &PlasmaWindow::movableChanged                  << qVariantFromValue(&PlasmaWindowInterface::setMovable)                  << QVariant(true);
+    QTest::newRow("resizable")        << &PlasmaWindow::resizableChanged                << qVariantFromValue(&PlasmaWindowInterface::setResizable)                << QVariant(true);
+    QTest::newRow("vdChangeable")     << &PlasmaWindow::virtualDesktopChangeableChanged << qVariantFromValue(&PlasmaWindowInterface::setVirtualDesktopChangeable) << QVariant(true);
+    QTest::newRow("onallDesktop")     << &PlasmaWindow::onAllDesktopsChanged            << qVariantFromValue(&PlasmaWindowInterface::setOnAllDesktops)            << QVariant(true);
+    QTest::newRow("title")            << &PlasmaWindow::titleChanged                    << qVariantFromValue(&PlasmaWindowInterface::setTitle)                    << QVariant(QStringLiteral("foo"));
+    QTest::newRow("appId")            << &PlasmaWindow::appIdChanged                    << qVariantFromValue(&PlasmaWindowInterface::setAppId)                    << QVariant(QStringLiteral("foo"));
+    QTest::newRow("icon" )            << &PlasmaWindow::iconChanged                     << qVariantFromValue(&PlasmaWindowInterface::setThemedIconName)           << QVariant(QStringLiteral("foo"));
+    QTest::newRow("vd")               << &PlasmaWindow::virtualDesktopChanged           << qVariantFromValue(&PlasmaWindowInterface::setVirtualDesktop)           << QVariant(2u);
+    QTest::newRow("unmapped")         << &PlasmaWindow::unmapped                        << qVariantFromValue(&PlasmaWindowInterface::unmap)                       << QVariant();
+}
+
+void PlasmaWindowModelTest::testChangeWindowAfterModelDestroy()
+{
+    // this test verifies that changes in a window after the model got destroyed doesn't crash
+    auto model = m_pw->createWindowModel();
+    QVERIFY(model);
+    QSignalSpy windowCreatedSpy(m_pw, &PlasmaWindowManagement::windowCreated);
+    QVERIFY(windowCreatedSpy.isValid());
+    auto w = m_pwInterface->createWindow(m_pwInterface);
+    QVERIFY(windowCreatedSpy.wait());
+    PlasmaWindow *window = windowCreatedSpy.first().first().value<PlasmaWindow*>();
+    // make sure the resource is properly created on server side
+    QCoreApplication::instance()->processEvents(QEventLoop::WaitForMoreEvents);
+    QCOMPARE(model->rowCount(), 1);
+    delete model;
+    QFETCH(ClientWindowSignal, changedSignal);
+    QSignalSpy changedSpy(window, changedSignal);
+    QVERIFY(changedSpy.isValid());
+    QVERIFY(!window->isActive());
+    QFETCH(QVariant, setter);
+    QFETCH(QVariant, value);
+    if (QMetaType::Type(value.type()) == QMetaType::Bool) {
+        (w->*(setter.value<ServerWindowBoolSetter>()))(value.toBool());
+    } else if (QMetaType::Type(value.type()) == QMetaType::QString) {
+        (w->*(setter.value<ServerWindowStringSetter>()))(value.toString());
+    } else if (QMetaType::Type(value.type()) == QMetaType::UInt) {
+        (w->*(setter.value<ServerWindowQuint32Setter>()))(value.toUInt());
+    } else if (!value.isValid()) {
+        (w->*(setter.value<ServerWindowVoidSetter>()))();
+    }
+
+    QVERIFY(changedSpy.wait());
+}
+
+void PlasmaWindowModelTest::testCreateWindowAfterModelDestroy()
+{
+    // this test verifies that creating a window after the model got destroyed doesn't crash
+    auto model = m_pw->createWindowModel();
+    QVERIFY(model);
+    delete model;
+    QSignalSpy windowCreatedSpy(m_pw, &PlasmaWindowManagement::windowCreated);
+    QVERIFY(windowCreatedSpy.isValid());
+    m_pwInterface->createWindow(m_pwInterface);
+    QVERIFY(windowCreatedSpy.wait());
+}
+
 QTEST_GUILESS_MAIN(PlasmaWindowModelTest)
 #include "test_plasma_window_model.moc"
