@@ -37,10 +37,12 @@ struct wl_subcompositor;
 struct wl_text_input_manager;
 struct zwp_text_input_manager_v2;
 struct _wl_fullscreen_shell;
+struct org_kde_kwin_appmenu_manager;
 struct org_kde_kwin_outputmanagement;
 struct org_kde_kwin_outputdevice;
 struct org_kde_kwin_fake_input;
 struct org_kde_kwin_idle;
+struct org_kde_kwin_remote_access_manager;
 struct org_kde_kwin_dpms_manager;
 struct org_kde_kwin_shadow_manager;
 struct org_kde_kwin_blur_manager;
@@ -49,14 +51,23 @@ struct org_kde_kwin_slide_manager;
 struct org_kde_plasma_shell;
 struct org_kde_plasma_window_management;
 struct org_kde_kwin_server_decoration_manager;
+struct org_kde_kwin_server_decoration_palette_manager;
 struct xdg_shell;
+struct zxdg_shell_v6;
 struct zwp_relative_pointer_manager_v1;
+struct zwp_pointer_gestures_v1;
+struct zwp_pointer_constraints_v1;
+struct zxdg_exporter_v2;
+struct zxdg_importer_v2;
+struct zwp_idle_inhibit_manager_v1;
+struct zxdg_output_manager_v1;
 
 namespace KWayland
 {
 namespace Client
 {
 
+class AppMenuManager;
 class Compositor;
 class ConnectionThread;
 class DataDeviceManager;
@@ -67,9 +78,13 @@ class FullscreenShell;
 class OutputManagement;
 class OutputDevice;
 class Idle;
+class IdleInhibitManager;
+class RemoteAccessManager;
 class Output;
 class PlasmaShell;
 class PlasmaWindowManagement;
+class PointerConstraints;
+class PointerGestures;
 class Seat;
 class ShadowManager;
 class BlurManager;
@@ -78,12 +93,18 @@ class SlideManager;
 class Shell;
 class ShmPool;
 class ServerSideDecorationManager;
+class ServerSideDecorationPaletteManager;
 class SubCompositor;
 class TextInputManager;
 class TextInputManagerUnstableV0;
 class TextInputManagerUnstableV2;
 class XdgShell;
 class RelativePointerManager;
+class XdgExporterUnstableV2;
+class XdgImporterUnstableV2;
+class XdgExporter;
+class XdgImporter;
+class XdgOutputManager;
 
 /**
  * @short Wrapper for the wl_registry interface.
@@ -144,7 +165,17 @@ public:
         TextInputManagerUnstableV0, ///< Refers to wl_text_input_manager, @since 5.23
         TextInputManagerUnstableV2, ///< Refers to zwp_text_input_manager_v2, @since 5.23
         XdgShellUnstableV5, ///< Refers to xdg_shell (unstable version 5), @since 5.25
-        RelativePointerManagerUnstableV1 ///< Refers to zwp_relative_pointer_manager_v1, @since 5.28
+        RelativePointerManagerUnstableV1, ///< Refers to zwp_relative_pointer_manager_v1, @since 5.28
+        PointerGesturesUnstableV1, ///< Refers to zwp_pointer_gestures_v1, @since 5.29
+        PointerConstraintsUnstableV1, ///< Refers to zwp_pointer_constraints_v1, @since 5.29
+        XdgExporterUnstableV2, ///< refers to zxdg_exporter_v2, @since 5.40
+        XdgImporterUnstableV2, ///< refers to zxdg_importer_v2, @since 5.40
+        XdgShellUnstableV6, ///< Refers to zxdg_shell_v6 (unstable version 6), @since 5.39
+        IdleInhibitManagerUnstableV1, ///< Refers to zwp_idle_inhibit_manager_v1 (unstable version 1), @since 5.41
+        AppMenu, ///Refers to org_kde_kwin_appmenu @since 5.42
+        ServerSideDecorationPalette, ///Refers to org_kde_kwin_server_decoration_palette_manager @since 5.42
+        RemoteAccessManager, ///< Refers to org_kde_kwin_remote_access_manager interface, @since 5.45
+        XdgOutputUnstableV1, ///refers to zxdg_output_v1, @since 5.47
     };
     explicit Registry(QObject *parent = nullptr);
     virtual ~Registry();
@@ -379,6 +410,16 @@ public:
      * @since 5.4
      **/
     org_kde_kwin_idle *bindIdle(uint32_t name, uint32_t version) const;
+    /**
+     * Binds the org_kde_kwin_remote_access_manager with @p name and @p version.
+     * If the @p name does not exist or is not for the idle interface,
+     * @c null will be returned.
+     *
+     * Prefer using createRemoteAccessManager instead.
+     * @see createRemoteAccessManager
+     * @since 5.45
+     **/
+    org_kde_kwin_remote_access_manager *bindRemoteAccessManager(uint32_t name, uint32_t version) const;
     /**
      * Binds the org_kde_kwin_fake_input with @p name and @p version.
      * If the @p name does not exist or is not for the fake input interface,
@@ -479,6 +520,16 @@ public:
      * @since 5.25
      **/
     xdg_shell *bindXdgShellUnstableV5(uint32_t name, uint32_t version) const;
+    /**
+     * Binds the zxdg_shell_v6 (unstable version 6) with @p name and @p version.
+     * If the @p name does not exist or is not for the xdg shell interface in unstable version 5,
+     * @c null will be returned.
+     *
+     * Prefer using createXdgShell instead.
+     * @see createXdgShell
+     * @since 5.39
+     **/
+    zxdg_shell_v6 *bindXdgShellUnstableV6(uint32_t name, uint32_t version) const;
     /**
      * Binds the zwp_relative_pointer_manager_v1 with @p name and @p version.
      * If the @p name does not exist or is not for the relative pointer interface in unstable version 1,
@@ -489,6 +540,92 @@ public:
      * @since 5.28
      **/
     zwp_relative_pointer_manager_v1 *bindRelativePointerManagerUnstableV1(uint32_t name, uint32_t version) const;
+    /**
+     * Binds the zwp_pointer_gestures_v1 with @p name and @p version.
+     * If the @p name does not exist or is not for the pointer gestures interface in unstable version 1,
+     * @c null will be returned.
+     *
+     * Prefer using createPointerGestures instead.
+     * @see createPointerGestures
+     * @since 5.29
+     **/
+    zwp_pointer_gestures_v1 *bindPointerGesturesUnstableV1(uint32_t name, uint32_t version) const;
+    /**
+     * Binds the zwp_pointer_constraints_v1 with @p name and @p version.
+     * If the @p name does not exist or is not for the pointer constraints interface in unstable version 1,
+     * @c null will be returned.
+     *
+     * Prefer using createPointerConstraints instead.
+     * @see createPointerConstraints
+     * @since 5.29
+     **/
+    zwp_pointer_constraints_v1 *bindPointerConstraintsUnstableV1(uint32_t name, uint32_t version) const;
+
+    /**
+     * Binds the zxdg_exporter_v2 with @p name and @p version.
+     * If the @p name does not exists or isnot for the exporter
+     * extension in unstable version 1,
+     * @c null will be returned.
+     *
+     * Prefer using createXdgExporter
+     * @since 5.40
+     */
+    zxdg_exporter_v2 *bindXdgExporterUnstableV2(uint32_t name, uint32_t version) const;
+
+    /**
+     * Binds the zxdg_importer_v2 with @p name and @p version.
+     * If the @p name does not exists or isnot for the importer
+     * extension in unstable version 1,
+     * @c null will be returned.
+     *
+     * Prefer using createXdgImporter
+     * @since 5.40
+     */
+    zxdg_importer_v2 *bindXdgImporterUnstableV2(uint32_t name, uint32_t version) const;
+
+    /**
+     * Binds the zwp_idle_inhibit_manager_v1 with @p name and @p version.
+     * If the @p name does not exists or is not for the idle inhibit manager in unstable version 1,
+     * @c null will be returned.
+     *
+     * Prefer using createIdleInhibitManager
+     * @since 5.41
+     */
+    zwp_idle_inhibit_manager_v1 *bindIdleInhibitManagerUnstableV1(uint32_t name, uint32_t version) const;
+
+    /**
+     * Binds the org_kde_kwin_appmenu_manager with @p name and @p version.
+     * If the @p name does not exist or is not for the appmenu manager interface,
+     * @c null will be returned.
+     *
+     * Prefer using createAppMenuManager instead.
+     * @see createAppMenuManager
+     * @since 5.42
+     **/
+    org_kde_kwin_appmenu_manager *bindAppMenuManager(uint32_t name, uint32_t version) const;
+
+    /**
+     * Binds the org_kde_kwin_server_decoration_palette_manager with @p name and @p version.
+     * If the @p name does not exist or is not for the server side decoration palette manager interface,
+     * @c null will be returned.
+     *
+     * Prefer using createServerSideDecorationPaletteManager instead.
+     * @see createServerSideDecorationPaletteManager
+     * @since 5.42
+     **/
+    org_kde_kwin_server_decoration_palette_manager *bindServerSideDecorationPaletteManager(uint32_t name, uint32_t version) const;
+
+    /**
+     * Binds the zxdg_output_v1 with @p name and @p version.
+     * If the @p name does not exist,
+     * @c null will be returned.
+     *
+     * Prefer using createXdgOutputManager instead.
+     * @see createXdgOutputManager
+     * @since 5.47
+     **/
+    zxdg_output_manager_v1 *bindXdgOutputUnstableV1(uint32_t name, uint32_t version) const;
+
     ///@}
 
     /**
@@ -696,6 +833,22 @@ public:
      * @since 5.4
      **/
     Idle *createIdle(quint32 name, quint32 version, QObject *parent = nullptr);
+    /**
+     * Creates a RemoteAccessManager and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * Note: in case @p name is invalid or isn't for the org_kde_kwin_remote_access_manager interface,
+     * the returned RemoteAccessManager will not be valid. Therefore it's recommended to call
+     * isValid on the created instance.
+     *
+     * @param name The name of the org_kde_kwin_remote_access_manager interface to bind
+     * @param version The version or the org_kde_kwin_remote_access_manager interface to use
+     * @param parent The parent for RemoteAccessManager
+     *
+     * @returns The created RemoteAccessManager.
+     * @since 5.45
+     **/
+    RemoteAccessManager *createRemoteAccessManager(quint32 name, quint32 version, QObject *parent = nullptr);
     /**
      * Creates a FakeInput and sets it up to manage the interface identified by
      * @p name and @p version.
@@ -863,8 +1016,142 @@ public:
      * @since 5.28
      **/
     RelativePointerManager *createRelativePointerManager(quint32 name, quint32 version, QObject *parent = nullptr);
+    /**
+     * Creates a PointerGestures and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * This factory method supports the following interfaces:
+     * @li zwp_pointer_gestures_v1
+     *
+     * If @p name is for one of the supported interfaces the corresponding manager will be created,
+     * otherwise @c null will be returned.
+     *
+     * @param name The name of the interface to bind
+     * @param version The version of the interface to use
+     * @param parent The parent for the PointerGestures
+     *
+     * @returns The created PointerGestures
+     * @since 5.29
+     **/
+    PointerGestures *createPointerGestures(quint32 name, quint32 version, QObject *parent = nullptr);
+    /**
+     * Creates a PointerConstraints and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * This factory method supports the following interfaces:
+     * @li zwp_pointer_constraints_v1
+     *
+     * If @p name is for one of the supported interfaces the corresponding manager will be created,
+     * otherwise @c null will be returned.
+     *
+     * @param name The name of the interface to bind
+     * @param version The version of the interface to use
+     * @param parent The parent for the PointerConstraints
+     *
+     * @returns The created PointerConstraints
+     * @since 5.29
+     **/
+    PointerConstraints *createPointerConstraints(quint32 name, quint32 version, QObject *parent = nullptr);
+
+    /**
+     * Creates an XdgExporter and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * This factory method supports the following interfaces:
+     * @li zxdg_exporter_v2
+     *
+     * If @p name is for one of the supported interfaces the corresponding manager will be created,
+     * otherwise @c null will be returned.
+     *
+     * @returns The created XdgExporter
+     * @since 5.40
+     */
+    XdgExporter *createXdgExporter(quint32 name, quint32 version, QObject *parent = nullptr);
+
+    /**
+     * Creates an XdgImporter and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * This factory method supports the following interfaces:
+     * @li zxdg_importer_v2
+     *
+     * If @p name is for one of the supported interfaces the corresponding manager will be created,
+     * otherwise @c null will be returned.
+     *
+     * @returns The created XdgImporter
+     * @since 5.40
+     */
+    XdgImporter *createXdgImporter(quint32 name, quint32 version, QObject *parent = nullptr);
+
+    /**
+     * Creates an IdleInhibitManager and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * This factory method supports the following interfaces:
+     * @li zwp_idle_inhibit_manager_v1
+     *
+     * If @p name is for one of the supported interfaces the corresponding manager will be created,
+     * otherwise @c null will be returned.
+     *
+     * @returns The created IdleInhibitManager
+     * @since 5.41
+     */
+    IdleInhibitManager *createIdleInhibitManager(quint32 name, quint32 version, QObject *parent = nullptr);
+
+    /**
+     * Creates a AppMenuManager and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * Note: in case @p name is invalid or isn't for the org_kde_kwin_appmenu_manager interface,
+     * the returned AppMenuManager will not be valid. Therefore it's recommended to call
+     * isValid on the created instance.
+     *
+     * @param name The name of the org_kde_kwin_appmenu_manager interface to bind
+     * @param version The version or the org_kde_kwin_appmenu_manager interface to use
+     * @param parent The parent for AppMenuManager
+     *
+     * @returns The created AppMenuManager.
+     * @since 5.42
+     **/
+    AppMenuManager *createAppMenuManager(quint32 name, quint32 version, QObject *parent = nullptr);
+
+    /**
+     * Creates a ServerSideDecorationPaletteManager and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * Note: in case @p name is invalid or isn't for the org_kde_kwin_appmenu_manager interface,
+     * the returned ServerSideDecorationPaletteManager will not be valid. Therefore it's recommended to call
+     * isValid on the created instance.
+     *
+     * @param name The name of the org_kde_kwin_server_decoration_palette_manager interface to bind
+     * @param version The version or the org_kde_kwin_server_decoration_palette_manager interface to use
+     * @param parent The parent for ServerSideDecorationPaletteManager
+     *
+     * @returns The created ServerSideDecorationPaletteManager.
+     * @since 5.42
+     **/
+    ServerSideDecorationPaletteManager *createServerSideDecorationPaletteManager(quint32 name, quint32 version, QObject *parent = nullptr);
+
+    /**
+     * Creates an XdgOutputManager and sets it up to manage the interface identified by
+     * @p name and @p version.
+     *
+     * Note: in case @p name is invalid or isn't for the zxdg_output_manager_v1 interface,
+     * the returned XdgOutputManager will not be valid. Therefore it's recommended to call
+     * isValid on the created instance.
+     *
+     * @param name The name of the zxdg_output_manager_v1 interface to bind
+     * @param version The version or the zxdg_output_manager_v1 interface to use
+     * @param parent The parent for XdgOuptutManager
+     *
+     * @returns The created XdgOuptutManager.
+     * @since 5.47
+     **/
+    XdgOutputManager *createXdgOutputManager(quint32 name, quint32 version, QObject *parent = nullptr);
+
     ///@}
 
+
     /**
      * cast operator to the low-level Wayland @c wl_registry
      **/
@@ -961,6 +1248,13 @@ Q_SIGNALS:
      * @since 5.4
      **/
     void idleAnnounced(quint32 name, quint32 version);
+    /**
+     * Emitted whenever a org_kde_kwin_remote_access_manager interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.45
+     **/
+    void remoteAccessManagerAnnounced(quint32 name, quint32 version);
     /**
      * Emitted whenever a org_kde_kwin_fake_input interface gets announced.
      * @param name The name for the announced interface
@@ -1031,6 +1325,14 @@ Q_SIGNALS:
      * @since 5.25
      **/
     void xdgShellUnstableV5Announced(quint32 name, quint32 version);
+    /**
+     * Emitted whenever a zxdg_shell_v6 (unstable version 6) interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.25
+     **/
+    void xdgShellUnstableV6Announced(quint32 name, quint32 version);
+
     /**
      * Emitted whenever a zwp_relative_pointer_manager_v1 interface gets announced.
      * @param name The name for the announced interface
@@ -1038,7 +1340,71 @@ Q_SIGNALS:
      * @since 5.28
      **/
     void relativePointerManagerUnstableV1Announced(quint32 name, quint32 version);
+    /**
+     * Emitted whenever a zwp_pointer_gestures_v1 interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.29
+     **/
+    void pointerGesturesUnstableV1Announced(quint32 name, quint32 version);
+    /**
+     * Emitted whenever a zwp_pointer_constraints_v1 interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.29
+     **/
+    void pointerConstraintsUnstableV1Announced(quint32 name, quint32 version);
+
+    /**
+     * Emitted whenever a zxdg_exporter_v2 interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.40
+     */
+    void exporterUnstableV2Announced(quint32 name, quint32 version);
+
+    /**
+     * Emitted whenever a zxdg_importer_v2 interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.40
+     */
+    void importerUnstableV2Announced(quint32 name, quint32 version);
+
+    /**
+     * Emitted whenever a zwp_idle_inhibit_manager_v1 interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.41
+     */
+    void idleInhibitManagerUnstableV1Announced(quint32 name, quint32 version);
+
+    /**
+     * Emitted whenever a org_kde_kwin_appmenu_manager interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.42
+     */
+    void appMenuAnnounced(quint32 name, quint32 version);
+
+    /**
+     * Emitted whenever a org_kde_kwin_server_decoration_palette_manager interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.42
+     */
+    void serverSideDecorationPaletteManagerAnnounced(quint32 name, quint32 version);
+
+    /**
+     * Emitted whenever a zxdg_output_v1 interface gets announced.
+     * @param name The name for the announced interface
+     * @param version The maximum supported version of the announced interface
+     * @since 5.47
+     */
+    void xdgOutputAnnounced(quint32 name, quint32 version);
+
     ///@}
+
     /**
      * @name Interface removed signals.
      **/
@@ -1113,6 +1479,12 @@ Q_SIGNALS:
      * @since 5.4
      **/
     void idleRemoved(quint32 name);
+    /**
+     * Emitted whenever a org_kde_kwin_remote_access_manager interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.45
+     **/
+    void remoteAccessManagerRemoved(quint32 name);
     /**
      * Emitted whenever a org_kde_kwin_fake_input interface gets removed.
      * @param name The name for the removed interface
@@ -1173,12 +1545,73 @@ Q_SIGNALS:
      * @since 5.25
      **/
     void xdgShellUnstableV5Removed(quint32 name);
+    /**
+     * Emitted whenever an xdg_shell (unstable version 5) interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.25
+     **/
+    void xdgShellUnstableV6Removed(quint32 name);
     /**
      * Emitted whenever a zwp_relative_pointer_manager_v1 interface gets removed.
      * @param name The name for the removed interface
      * @since 5.28
      **/
     void relativePointerManagerUnstableV1Removed(quint32 name);
+    /**
+     * Emitted whenever a zwp_pointer_gestures_v1 interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.29
+     **/
+    void pointerGesturesUnstableV1Removed(quint32 name);
+    /**
+     * Emitted whenever a zwp_pointer_constraints_v1 interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.29
+     **/
+    void pointerConstraintsUnstableV1Removed(quint32 name);
+
+    /**
+     * Emitted whenever a zxdg_exporter_v2 interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.40
+     **/
+    void exporterUnstableV2Removed(quint32 name);
+
+    /**
+     * Emitted whenever a zxdg_importer_v2 interface gets removed.
+     * @param name The name for the removed interface
+     * @since 5.40
+     **/
+    void importerUnstableV2Removed(quint32 name);
+
+    /**
+     * Emitted whenever a zwp_idle_inhibit_manager_v1 interface gets removed.
+     * @param name The name of the removed interface
+     * @since 5.41
+     **/
+    void idleInhibitManagerUnstableV1Removed(quint32 name);
+
+    /**
+     * Emitted whenever a org_kde_kwin_appmenu_manager gets removed.
+     * @param name The name of the removed interface
+     * @since 5.42
+     **/
+    void appMenuRemoved(quint32 name);
+
+    /**
+     * Emitted whenever a org_kde_kwin_server_decoration_palette_manager gets removed.
+     * @param name The name of the removed interface
+     * @since 5.42
+     **/
+    void serverSideDecorationPaletteManagerRemoved(quint32 name);
+
+    /**
+     * Emitted whenever a zxdg_output_v1 gets removed.
+     * @param name The name of the removed interface
+     * @since 5.47
+     **/
+    void xdgOutputRemoved(quint32 name);
+
     ///@}
     /**
      * Generic announced signal which gets emitted whenever an interface gets
@@ -1212,6 +1645,12 @@ Q_SIGNALS:
      **/
     void interfacesAnnounced();
 
+Q_SIGNALS:
+    /*
+     * Emitted when the registry has been destroyed rather than released
+     */
+    void registryDestroyed();
+
 private:
     class Private;
     QScopedPointer<Private> d;
