@@ -108,7 +108,9 @@ public:
         Normal, ///< A normal surface
         Desktop, ///< The surface represents a desktop, normally stacked below all other surfaces
         Panel, ///< The surface represents a panel (dock), normally stacked above normal surfaces
-        OnScreenDisplay ///< The surface represents an on screen display, like a volume changed notification
+        OnScreenDisplay, ///< The surface represents an on screen display, like a volume changed notification
+        Notification, ///< The surface represents a notification @since 5.24
+        ToolTip ///< The surface represents a tooltip @since 5.24
     };
     /**
      * @returns The requested role, default value is @c Role::Normal.
@@ -133,9 +135,47 @@ public:
      * @returns true if this window doesn't want to be listed
      * in the taskbar
      * @since 5.5
-     */
+     **/
     bool skipTaskbar() const;
 
+    /**
+     * @returns true if this window doesn't want to be listed
+     * in a window switcher
+     * @since 5.45
+     **/
+    bool skipSwitcher() const;
+
+    /**
+     * Informs the PlasmaShellSurfaceInterface that the auto-hiding panel got hidden.
+     * Once it is shown again the method {@link showAutoHidingPanel} should be used.
+     *
+     * @see showAutoHidingPanel
+     * @see panelAutoHideHideRequested
+     * @see panelAutoHideShowRequested
+     * @since 5.28
+     **/
+    void hideAutoHidingPanel();
+
+    /**
+     * Informs the PlasmaShellSurfaceInterface that the auto-hiding panel got shown again.
+     *
+     * @see hideAutoHidingPanel
+     * @see panelAutoHideHideRequested
+     * @see panelAutoHideShowRequested
+     * @see 5.28
+     **/
+    void showAutoHidingPanel();
+
+    /**
+     * Whether a PlasmaShellSurfaceInterface with Role Panel wants to have focus.
+     *
+     * By default a Panel does not get focus, but the PlasmaShellSurfaceInterface can
+     * request that it wants to have focus. The compositor can use this information to
+     * pass focus to the panel.
+     * @since 5.28
+     **/
+    bool panelTakesFocus() const;
+
     /**
      * @returns The PlasmaShellSurfaceInterface for the @p native resource.
      * @since 5.5
@@ -159,6 +199,39 @@ Q_SIGNALS:
      * A change in the skip taskbar property has been requested
      */
     void skipTaskbarChanged();
+    /**
+     * A change in the skip switcher property has been requested
+     **/
+    void skipSwitcherChanged();
+
+    /**
+     * A surface with Role Panel and PanelBehavior AutoHide requested to be hidden.
+     *
+     * The compositor should inform the PlasmaShellSurfaceInterface about the actual change.
+     * Once the surface is hidden it should invoke {@link hideAutoHidingPanel}. If the compositor
+     * cannot hide the surface (e.g. because it doesn't border a screen edge) it should inform
+     * the surface through invoking {@link showAutoHidingPanel}. This method should also be invoked
+     * whenever the surface gets shown again due to triggering the screen edge.
+     *
+     * @see hideAutoHidingPanel
+     * @see showAutoHidingPanel
+     * @see panelAutoHideShowRequested
+     * @since 5.28
+     **/
+    void panelAutoHideHideRequested();
+
+    /**
+     * A surface with Role Panel and PanelBehavior AutoHide requested to be shown.
+     *
+     * The compositor should inform the PlasmaShellSurfaceInterface about the actual change.
+     * Once the surface is shown it should invoke {@link showAutoHidingPanel}.
+     *
+     * @see hideAutoHidingPanel
+     * @see showAutoHidingPanel
+     * @see panelAutoHideHideRequested
+     * @since 5.28
+     **/
+    void panelAutoHideShowRequested();
 
 private:
     friend class PlasmaShellInterface;
@@ -170,4 +243,7 @@ private:
 }
 }
 
+Q_DECLARE_METATYPE(KWayland::Server::PlasmaShellSurfaceInterface::Role)
+Q_DECLARE_METATYPE(KWayland::Server::PlasmaShellSurfaceInterface::PanelBehavior)
+
 #endif
